from enum import Enum


class Language(Enum):
    EN = "EN"
    RU = "RU"
    KO = "KO"
    CN = "CN"
    FR = "FR"
    PL = "PL"
    ES = "ES"
    TH = "TH"
    JP = "JP"
    DE = "DE"
    IT = "IT"
    UA = "UA"
    PT = "PT"
    BR = PT
    CZ = "CZ"
